/*
 * Say ioctl to the hamachi.
 * 2/19/99 Pete Wyckoff <wyckoff@ca.sandia.gov>
 *
 * Watch out for the cheesy way I "detect" the card by looking at
 * IP numbers.  A better way would be to know in advance which device
 * it is, such as "eth1" and hardcode that into a comparison on the
 * name field, or supply it on the command line.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <net/if.h>
#include <netinet/in.h>

typedef unsigned int u32;

static void
usage(void)
{
    fprintf(stderr, "Usage: setintr { -d | <tx> <rx> }  (values in hex)\n");
    exit(1);
}

int
main(int argc, char* argv[])
{
    int i, s;
    u32 tx = 0, rx = 0, *d;
    struct ifconf ifc;
    struct ifreq *ifr = NULL;
    struct sockaddr_in *sin;
    char buf[128];

    if (argc == 2 && !strcmp(argv[1], "-d")) {
	tx = 0x00080000;  /* "defaults" from the driver" */
	rx = 0x00000020;
    } else if (argc == 3) {
	tx = strtoul(argv[1], 0, 16);
	rx = strtoul(argv[2], 0, 16);
    } else
    usage();

    if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("socket");
	return 1;
    }

    ifc.ifc_buf = buf;
    ifc.ifc_len = sizeof(buf);
    if (ioctl(s, SIOCGIFCONF, &ifc) < 0) {
	perror("ioctl SIOCGIFCONF");
	return 1;
    }

    ifc.ifc_len /= sizeof(struct ifreq);
    for (i=0; i<ifc.ifc_len; i++) {
	ifr = (struct ifreq *) &ifc.ifc_req[i];
	sin = (struct sockaddr_in *) &ifr->ifr_addr;
	if (sin->sin_family == AF_INET) {
	    if ((sin->sin_addr.s_addr & 0xffff) == 0x020a)
		break;  /* 10.2.something */
	}
    }
    if (i == ifc.ifc_len) {
	printf("not found\n");
	return 0;
    }

    d = (u32 *)&ifr->ifr_data;
    d[0] = tx;
    d[1] = rx;
    if (ioctl(s, SIOCDEVPRIVATE+3, ifr) < 0) {
	perror("ioctl SIOCDEVPRIVATE+3");
	return 1;
    }
    return 0;
}

